@@ -183,15 +183,19 @@ void find_bounding_box(NET net)
 
       n1 = (NODE)net->netnodes;
       n2 = (NODE)net->netnodes->next;
-      mindist = MAXRT;
 
       // Simple 2-pass---pick up first tap on n1, find closest tap on n2,
       // then find closest tap on n1.
+
       d1tap = (n1->taps == NULL) ? n1->extend : n1->taps;
       if (d1tap == NULL) return;
       d2tap = (n2->taps == NULL) ? n2->extend : n2->taps;
       if (d2tap == NULL) return;
-      for (; d2tap != NULL; d2tap = d2tap->next) {
+      dx = d2tap->gridx - d1tap->gridx;
+      dy = d2tap->gridy - d1tap->gridy;
+      mindist = dx * dx + dy * dy;
+      mintap = d2tap;
+      for (d2tap = d2tap->next; d2tap != NULL; d2tap = d2tap->next) {
          dx = d2tap->gridx - d1tap->gridx;
          dy = d2tap->gridy - d1tap->gridy;
          dist = dx * dx + dy * dy;
@@ -201,9 +205,12 @@ void find_bounding_box(NET net)
          }
       }
       d2tap = mintap;
-      mindist = MAXRT;
-      for (d1tap = (n1->taps == NULL) ? n1->extend : n1->taps; d1tap != NULL;
-                d1tap = d1tap->next) {
+      d1tap = (n1->taps == NULL) ? n1->extend : n1->taps;
+      dx = d2tap->gridx - d1tap->gridx;
+      dy = d2tap->gridy - d1tap->gridy;
+      mindist = dx * dx + dy * dy;
+      mintap = d1tap;
+      for (d1tap = d1tap->next; d1tap != NULL; d1tap = d1tap->next) {
          dx = d2tap->gridx - d1tap->gridx;
          dy = d2tap->gridy - d1tap->gridy;
          dist = dx * dx + dy * dy;
@@ -279,7 +286,7 @@ void defineRouteTree(NET net)
 	net->trunkx = xmin;
 	net->trunky = ymin;
     }
-    else {
+    else if (net->numnodes > 0) {
 
 	// Use the first tap point for each node to get a rough
 	// centroid of all taps
@@ -501,12 +508,50 @@ void create_obstructions_from_variable_pitch()
 void
 disable_gridpos(int x, int y, int lay)
 {
-   int apos = OGRID(x, y, lay);
+    int apos = OGRID(x, y, lay);
 
-   Obs[lay][apos] = (u_int)(NO_NET | OBSTRUCT_MASK);
-   Nodeloc[lay][apos] = NULL;
-   Nodesav[lay][apos] = NULL;
-   Stub[lay][apos] = 0.0;
+    Obs[lay][apos] = (u_int)(NO_NET | OBSTRUCT_MASK);
+    Nodeloc[lay][apos] = NULL;
+    Nodesav[lay][apos] = NULL;
+    Stub[lay][apos] = 0.0;
+}
+
+/*--------------------------------------------------------------*/
+/* count_pinlayers()---						*/
+/*	Check which layers have non-NULL Nodeloc, Nodesav, and	*/
+/* 	Stub entries.  Then set "Pinlayers" and free all the	*/
+/*	unused layers.  This saves a lot of memory, especially	*/
+/*	when the number of routing layers becomes large.	*/ 
+/*--------------------------------------------------------------*/
+
+void
+count_pinlayers()
+{
+   int x, y, l, haspin;
+
+   Pinlayers = 0;
+   for (l = 0; l < Num_layers; l++) {
+      haspin = 0;
+      for (x = 0; x < NumChannelsX[l]; x++) {
+	 for (y = 0; y < NumChannelsY[l]; y++) {
+	    if (Nodesav[l][OGRID(x, y, l)] != NULL) {
+	       haspin = 1;
+	       Pinlayers = l + 1;
+	       break;
+	    }
+	 }
+	 if (haspin) break;
+      }
+   }
+
+   for (l = Pinlayers; l < Num_layers; l++) {
+      free(Stub[l]);
+      free(Nodeloc[l]);
+      free(Nodesav[l]);
+      Stub[l] = NULL;
+      Nodeloc[l] = NULL;
+      Nodesav[l] = NULL;
+   }
 }
 
 /*--------------------------------------------------------------*/
